package com.book.push.handler;

import com.alibaba.fastjson.JSON;
import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.User;
import com.book.dao.cps.pojo.WechatAutoreply;
import com.book.push.builder.TextBuilder;
import com.book.push.builder.TuWenBuilder;
import com.book.push.cons.UrlCons;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.OpHostService;
import com.book.push.service.dao.UserService;
import com.book.push.service.dao.WechatAutoreplayService;
import com.book.push.utils.JsonUtils;
import com.book.push.vo.NewsMsgContent;
import com.book.push.vo.TextMsgContent;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.common.session.WxSessionManager;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutNewsMessage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static me.chanjar.weixin.common.api.WxConsts.XmlMsgType;

/**
 * @author
 */
@Component
public class MsgHandler extends AbstractHandler {
    @Autowired
    private WechatAutoreplayService wechatAutoreplayService;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private UserService userService;
    @Autowired
    private OpHostService opHostService;

    @Override
    public WxMpXmlOutMessage handle(WxMpXmlMessage wxMessage,
                                    Map<String, Object> context, WxMpService weixinService,
                                    WxSessionManager sessionManager) {


        AdminConfig adminConfig = adminConfigService.selectByAppid(wxMessage.getToUser());
        WechatAutoreply wechatAutoreply = wechatAutoreplayService.selectByAdminIdAndText(adminConfig.getAdminId(), wxMessage.getContent());
        User user = userService.selectByOpenId(wxMessage.getFromUser());
        String host = opHostService.selectById(adminConfig.getOphostId()).getHost();
        if (wechatAutoreply != null) {
            String type = wechatAutoreply.getType();
            if ("text".equals(type)) {
                String textContent = wechatAutoreply.getTextContent();
                List<TextMsgContent> textMsgContents = JsonUtils.getList(textContent, TextMsgContent.class);
                StringBuilder sb = new StringBuilder();
                for (TextMsgContent textMsgContent : textMsgContents) {
                    sb.append(String.format("<a href='%s'>%s</a>\n", textMsgContent.getUrl().replace("{$ophost}", host), textMsgContent.getTitle()));
                }
                String nickname = user.getNickname();
                String json = adminConfig.getJson().toString();
                AdminConfig.Info info = JsonUtils.getObject(json, AdminConfig.Info.class);
                String nick_name = info.getAuthorizer_info().getNick_name();
                String msg = "@%s，欢迎关注 [%s]，点击下方继续阅读 %s\n\r为了方便下次阅读，请置顶公众号";
                msg = String.format(msg, nickname, nick_name, sb.toString());
                return new TextBuilder().build(msg, wxMessage, weixinService);
            } else if ("news".equals(type)) {

                List<NewsMsgContent> newsMsgContents = JsonUtils.getList(wechatAutoreply.getNewsContent(), NewsMsgContent.class);
                List<WxMpXmlOutNewsMessage.Item> items = new ArrayList<>();
                for (NewsMsgContent newsMsgContent : newsMsgContents) {
                    WxMpXmlOutNewsMessage.Item item = new WxMpXmlOutNewsMessage.Item();
                    item.setTitle(newsMsgContent.getTitle());
                    item.setPicUrl(newsMsgContent.getImage());
                    item.setDescription(newsMsgContent.getDescription());
                    String url = newsMsgContent.getUrl().replace("{$ophost}", host);
                    if (!StringUtils.isEmpty(newsMsgContent.getBook_id())) {
                        url = String.format(UrlCons.FRONT_URL, adminConfig.getAppid(), host, UrlCons.MODEL_BOOK, adminConfig.getAdminId(), user.getId() + "")
                                .concat("&bookId=").concat(newsMsgContent.getBook_id()).concat("&").concat(newsMsgContent.getChapter_id());
                    }
                    item.setUrl(url);
                }

                new TuWenBuilder().build(items, wxMessage, weixinService);
            }

        }


        return new TextBuilder().build("success", wxMessage, weixinService);

    }

}
