package com.book.push.service.dao.impl;

import com.book.dao.cps.mapper.RechargeMapper;
import com.book.dao.cps.mapper.UserMapper;
import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Recharge;
import com.book.dao.cps.pojo.User;
import com.book.dao.cps.pojo.example.RechargeExample;
import com.book.dao.utils.DateUtils;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.UserService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * created in 2021/8/17
 * Project: book-store
 *
 * @author win7
 */
@Service
public class UserServiceImpl implements UserService {
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private RechargeMapper rechargeMapper;

    @Override
    public User selectUserByUser(User user) {
        return userMapper.selectByUser(user);
    }

    @Override
    public User selectByOpenId(String openId) {
        return userMapper.selectByOpenId(openId);
    }

    @Override
    public User selectById(Long id) {
        return userMapper.selectByPrimaryKey(id);
    }

    @Override
    public User createUserByWxUser(WxMpUser wxMpUser, int channel_id) {

        User user = new User();
        user.setOpenid(wxMpUser.getOpenId());
        user.setArea(wxMpUser.getCity());
        user.setCity(wxMpUser.getCity());
        user.setCountry(wxMpUser.getCountry());
        user.setUnionid(wxMpUser.getUnionId());
        user.setNickname(wxMpUser.getNickname());
        user.setSex(wxMpUser.getSex());
//        user.setMobile(wxMpUser.getmobile);
        user.setAvatar(wxMpUser.getHeadImgUrl());
        user.setSubscribeTime(Integer.valueOf(String.valueOf(System.currentTimeMillis() / 1000)));
        user.setOperateTime(user.getSubscribeTime());
//        user.setRegisterIp(wxMpUser.geti);
        user.setSubscriptionExtend("1");
        user.setProvince(wxMpUser.getProvince());
        user.setCreatetime(Integer.valueOf(String.valueOf(System.currentTimeMillis() / 1000)));
        user.setChannelId(channel_id);

        userMapper.insert(user);
        return user;
    }

    @Override
    public AdminConfig selectAdminConfigByOpenid(String openid) {
        User user = userMapper.selectByOpenId(openid);
        Integer channelId = user.getChannelId();
        AdminConfig adminConfig = adminConfigService.selectByAdminId(channelId);
        return adminConfig;
    }

    @Override
    public List<User> selectListByPushMap(Map map) {

        return userMapper.selectListByPushMap(map);
    }

    @Override
    public void updateOperateTimeByOpenid(String openid, Integer operateTime) {
        userMapper.updateOperateTimeByOpenid(openid, operateTime);
    }

    @Override
    public int insertKandian(Long userId, String type, int kandian, int freeKandian) {

        // 签到看点
        rechargeMapper.insertSelective(
                Recharge.builder()
                        .userId(userId)
                        .type(type)
                        .kandian(kandian)
                        .freeKandian(freeKandian)
                        .remainFreeKandian(freeKandian)
                        .freeEndtime(DateUtils.get3DayAfter())
                        .createtime(DateUtils.getNow())
                        .updatetime(DateUtils.getNow())
                        .build()
        );
        int freeCount = getFreeCountFromCharges(getFreeCharges(userId));

        // 更新到用户
        User user = selectById(userId);
        User user1 = new User();
        user1.setKandian(user.getKandian() + kandian);
        user1.setFreeKandian(freeCount);
        int i = userMapper.updateByPrimaryKeySelective(user1);

        return i;
    }

    public List<Recharge> getFreeCharges(Long userId) {
        RechargeExample example = RechargeExample.newAndCreateCriteria()
                .andUserIdEqualTo(userId)
                .andFreeEndtimeGreaterThan(DateUtils.getNow())
                .andRemainFreeKandianGreaterThan(0)
                .example();
        example.orderBy(Recharge.Column.freeEndtime.desc());
        List<Recharge> freeRecharges = rechargeMapper.selectByExample(example);
        return freeRecharges;
    }

    public int getFreeCountFromCharges(List<Recharge> freeRecharges) {
        Integer freeCount = 0;
        if (freeRecharges != null) {
            freeCount = freeRecharges.stream().mapToInt(x -> x.getFreeKandian()).sum();
        }
        return freeCount;
    }
}
