package com.book.server.service;

import com.book.dao.VO.BlockRes;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

@Component
public class BlockCacheGuava {
    private LoadingCache<Integer, List<BlockRes>> blockResCache;

    @Autowired
    private CacheService cacheService;

    @PostConstruct
    public void init() {
        blockResCache = CacheBuilder.newBuilder()
                .maximumSize(10) //设置缓存最大容量
                .refreshAfterWrite(5, TimeUnit.MINUTES)
                .build(new CacheLoader<Integer, List<BlockRes>>() {
                    @Override
                    public List<BlockRes> load(Integer key) throws Exception {
                        return cacheService.getBlockByPageId(key);
                    }
                });
    }

    public List<BlockRes> getBlockRes(Integer key) throws ExecutionException {
        return blockResCache.get(key);
    }

    public void clearCache() {
        blockResCache.refresh(1);
        blockResCache.refresh(2);
        cacheService.deleteSmartRecommandCache("1");
        cacheService.deleteSmartRecommandCache("2");
        cacheService.deleteTopRecommendCache("1");
        cacheService.deleteTopRecommendCache("2");
        cacheService.deleteSearchKeywordsCache("1");
        cacheService.deleteSearchKeywordsCache("2");
        cacheService.deleteRecommendFromCache("1",0,10);
        cacheService.deleteRecommendFromCache("1",1,10);
        cacheService.deleteRecommendFromCache("2",0,10);
        cacheService.deleteRecommendFromCache("2",1,10);
    }
}
